function finished = ExtractConvnet(convnetFile, imagePath, ...
    descriptor1Path, descriptor2Path, gradientPath, featurePath, layer, pyramid)

convnet = load(convnetFile);
stride = 1;
if (layer == 28 || layer == 30 || layer == 32 || layer == 34)
    window = 3; pad = 1;
elseif (layer == 37)
    window = 7; pad = 0;
elseif (layer == 39)
    window = 1; pad = 0;
end

feature = [];
feature.categories = 0;
feature.images = 0;

if (exist(imagePath, 'dir'))
    fprintf('Begin to extract ConvNet features.\n');
    categoryDirectories = dir(imagePath);
    for i = 1: 1: length(categoryDirectories)
        categoryName = categoryDirectories(i).name;
        if (~strcmp(categoryName, '.') && ~strcmp(categoryName, '..'))
            feature.categories = feature.categories + 1;
            fprintf('Category # %03d: %s, begins.\n', feature.categories, categoryName);
            ca = feature.categories;
            feature.cate(ca).images = 0;
            imageDirectory = fullfile(imagePath, categoryName);
            imageFiles = dir(fullfile(imageDirectory, '*.jpg'));
            descriptor1Directory = fullfile(descriptor1Path, categoryName);
            if (~isdir(descriptor1Directory))
                mkdir(descriptor1Directory);
            end
            descriptor2Directory = fullfile(descriptor2Path, categoryName);
            if (~isdir(descriptor2Directory))
                mkdir(descriptor2Directory);
            end
            gradientDirectory = fullfile(gradientPath, categoryName);
            if (~isdir(gradientDirectory))
                mkdir(gradientDirectory);
            end
            featureDirectory = fullfile(featurePath, categoryName);
            if (~isdir(featureDirectory))
                mkdir(featureDirectory);
            end
            for j = 1: 1: length(imageFiles)
                feature.cate(ca).images = feature.cate(ca).images + 1;
                imageFile = fullfile(imageDirectory, imageFiles(j).name);
                [~, name_] = fileparts(imageFiles(j).name);
                descriptor1File = fullfile(descriptor1Directory, [name_, '.mat']);
                descriptor2File = fullfile(descriptor2Directory, [name_, '.mat']);
                gradientFile = fullfile(gradientDirectory, [name_, '.mat']);
                featureFile = fullfile(featureDirectory, [name_, '.mat']);
                if (~exist(featureFile, 'file'))
                    if (~exist(descriptor1File, 'file') || ~exist(descriptor2File, 'file'))
                        image = imread(imageFile);
                        result = DeepFeature(image, convnet);
                        descriptors = gather(result(layer + 1).x);
                        descriptors1 = descriptors;
                        save(descriptor1File, 'descriptors');
                        descriptors = gather(result(layer + 3).x);
                        descriptors2 = descriptors;
                        save(descriptor2File, 'descriptors');
                        gradients = gather(result(layer + 3).dzdx);
                        save(gradientFile, 'gradients');
                    else
                        descriptors1 = load(descriptor1File);
                        descriptors1 = descriptors1.descriptors;
                        descriptors2 = load(descriptor2File);
                        descriptors2 = descriptors2.descriptors;
                        gradients = load(gradientFile);
                        gradients = gradients.gradients;
                    end
                    [height1, width1, channels_] = size(descriptors1);
                    weight = zeros(height1 + pad * 2, width1 + pad * 2);
                    [height2, width2, ~] = size(descriptors2);
                    for h = 1: 1: height2
                        for w = 1: 1: width2
                            weight_ = sum(gradients(h, w, :) .* (descriptors2(h, w, :) > 0));
                            weight((h - 1) * stride + 1: (h - 1) * stride + window, ...
                                (w - 1) * stride + 1: (w - 1) * stride + window) = ...
                                weight((h - 1) * stride + 1: (h - 1) * stride + window, ...
                                (w - 1) * stride + 1: (w - 1) * stride + window) + weight_;
                        end
                    end
                    weight = weight(pad + 1: height1 + pad, pad + 1: width1 + pad);
                    descriptors = descriptors1 .* repmat(weight, [1, 1, channels_]);
                    features = [];
                    for p = 1: 1: size(pyramid, 1)
                        for f1 = 1: 1: pyramid(p, 1)
                            for f2 = 1: 1: pyramid(p, 2)
                                topY = max(floor(height1 / pyramid(p, 1) * (f1 - 1)), 0) + 1;
                                bottomY = min(ceil(height1 / pyramid(p, 1) * f1), height1);
                                leftX = max(floor(width1 / pyramid(p, 2) * (f2 - 1)), 0) + 1;
                                rightX = min(ceil(width1 / pyramid(p, 2) * f2), width1);
                                features_ = descriptors(topY: bottomY, leftX: rightX, :);
                                features_ = squeeze(mean(mean(features_, 1), 2));
                                features_ = sign(features_) .* sqrt(abs(features_));
                                features = cat(2, features, features_');
                            end
                        end
                    end
                    features = features / ((sum(features .^ 2)) ^ (1 / 2)); %#ok<NASGU>
                    save(featureFile, 'features');
                    fprintf('    Image #%03d out of %d images of category %03d is finished.\n', ...
                        j, length(imageFiles), ca);
                else
                    
                end
            end
            feature.images = feature.images + feature.cate(ca).images;
            fprintf('Category # %03d: %s, is finished, %04d images have been processed.\n', ...
                feature.categories, categoryName, feature.images);
        end
    end
    fprintf('Extraction of ConvNet features is finished.\n');
    finished = true;
else
    fprintf('Bad image directory. Please check if %s exists.\n', imagePath);
    finished = false;
end

end
