function finished = ExtractConvnet(convnetFile, imagePath, featurePath, tail, pyramid)

convnet = load(convnetFile);

feature = [];
feature.categories = 0;
feature.images = 0;

if (exist(imagePath, 'dir'))
    fprintf('Begin to extract ConvNet features.\n');
    categoryDirectories = dir(imagePath);
    for i = 1: 1: length(categoryDirectories)
        categoryName = categoryDirectories(i).name;
        if (~strcmp(categoryName, '.') && ~strcmp(categoryName, '..'))
            feature.categories = feature.categories + 1;
            fprintf('Category # %03d: %s, begins.\n', feature.categories, categoryName);
            ca = feature.categories;
            feature.cate(ca).images = 0;
            imageDirectory = fullfile(imagePath, categoryName);
            imageFiles = dir(fullfile(imageDirectory, '*.jpg'));
            featureDirectory = fullfile(featurePath, categoryName);
            if (~isdir(featureDirectory))
                mkdir(featureDirectory);
            end
            for j = 1: 1: length(imageFiles)
                feature.cate(ca).images = feature.cate(ca).images + 1;
                imageFile = fullfile(imageDirectory, imageFiles(j).name);
                [~, name_] = fileparts(imageFiles(j).name);
                featureFile = fullfile(featureDirectory, [name_, '.mat']);
                if (~exist(featureFile, 'file'))
                    image = imread(imageFile);
                    if (size(image, 3) == 1)
                        image = repmat(image, [1, 1, 3]);
                    end
                    [height, width, ~] = size(image);
                    features = [];
                    for p = 1: 1: size(pyramid, 1)
                        for s1 = 1: 1: pyramid(p, 1)
                            for s2 = 1: 1: pyramid(p, 2)
                                topY = max(floor(height / pyramid(p, 1) * (s1 - 1)), 0) + 1;
                                bottomY = min(ceil(height / pyramid(p, 1) * s1), height);
                                leftX = max(floor(width / pyramid(p, 2) * (s2 - 1)), 0) + 1;
                                rightX = min(ceil(width / pyramid(p, 2) * s2), width);
                                image_ = image(topY: bottomY, leftX: rightX, :);
                                image_ = single(image_);
                                image_ = imresize(image_, convnet.normalization.imageSize(1:2));
                                image_ = image_ - convnet.normalization.averageImage;
                                result = vl_simplenn(convnet, image_);
                                features_ = squeeze(gather(result(end - tail).x));
                                features = cat(3, features, features_);
                            end
                        end
                    end
                    %features = sign(features) .* sqrt(abs(features));
                    %features = features / ((sum(features .^ 2)) ^ (1 / 2)); %#ok<NASGU>
                    save(featureFile, 'features');
                    fprintf('    Image #%03d out of %d images of category %03d is finished.\n', ...
                        j, length(imageFiles), ca);
                else
                    
                end
            end
            feature.images = feature.images + feature.cate(ca).images;
            fprintf('Category # %03d: %s, is finished, %04d images have been processed.\n', ...
                feature.categories, categoryName, feature.images);
        end
    end
    fprintf('Extraction of ConvNet features is finished.\n');
    finished = false;
else
    fprintf('Bad image directory. Please check if %s exists.\n', imagePath);
    finished = false;
end

end
