function finished = ExtractConvnet(convnetFile, imagePath, featurePath)

convnet = load(convnetFile);

feature = [];
feature.categories = 0;
feature.images = 0;

if (exist(imagePath, 'dir'))
    fprintf('Begin to extract ConvNet features.\n');
    categoryDirectories = dir(imagePath);
    for i = 1: 1: length(categoryDirectories)
        categoryName = categoryDirectories(i).name;
        if (~strcmp(categoryName, '.') && ~strcmp(categoryName, '..'))
            feature.categories = feature.categories + 1;
            fprintf('Category # %03d: %s, begins.\n', feature.categories, categoryName);
            ca = feature.categories;
            feature.cate(ca).images = 0;
            imageDirectory = fullfile(imagePath, categoryName);
            imageFiles = dir(fullfile(imageDirectory, '*.jpg'));
            for j = 1: 1: length(imageFiles)
                feature.cate(ca).images = feature.cate(ca).images + 1;
                imageFile = fullfile(imageDirectory, imageFiles(j).name);
                image = imread(imageFile);
                result = DeepFeature(image, convnet);
                for layer = [5, 10, 19, 28, 37, 39, 41, 42]
                    for pyramid = 1: 1: 2
                        featurePath_ = [featurePath, ',', num2str(layer), 'L_', num2str(pyramid), 'L'];
                        featureDirectory = fullfile(featurePath_, categoryName);
                        if (~isdir(featureDirectory))
                            mkdir(featureDirectory);
                        end
                        [~, name_] = fileparts(imageFiles(j).name);
                        featureFile = fullfile(featureDirectory, [name_, '.mat']);
                        descriptors = gather(result(layer + 1).x);
                        [height_, width_, ~] = size(descriptors);
                        features = [];
                        for p = 1: 1: pyramid
                            for f1 = 1: 1: 2 ^ (p - 1)
                                for f2 = 1: 1: 2 ^ (p - 1)
                                    topY = max(floor(height_ / 2 ^ (p - 1) * (f1 - 1)), 0) + 1;
                                    bottomY = min(ceil(height_ / 2 ^ (p - 1) * f1), height_);
                                    leftX = max(floor(width_ / 2 ^ (p - 1) * (f2 - 1)), 0) + 1;
                                    rightX = min(ceil(width_ / 2 ^ (p - 1) * f2), width_);
                                    features_ = descriptors(topY: bottomY, leftX: rightX, :);
                                    features_ = squeeze(mean(mean(features_, 1), 2));
                                    features_ = sign(features_) .* sqrt(abs(features_));
                                    features = cat(2, features, features_');
                                end
                            end
                        end
                        features = features / ((sum(features .^ 2)) ^ (1 / 2)); %#ok<NASGU>
                        save(featureFile, 'features');
                    end
                end
                fprintf('    Image #%03d out of %d images of category %03d is finished.\n', ...
                    j, length(imageFiles), ca);
            end
            feature.images = feature.images + feature.cate(ca).images;
            fprintf('Category # %03d: %s, is finished, %04d images have been processed.\n', ...
                feature.categories, categoryName, feature.images);
        end
    end
    fprintf('Extraction of ConvNet features is finished.\n');
    finished = false;
else
    fprintf('Bad image directory. Please check if %s exists.\n', imagePath);
    finished = false;
end

end
