function result = DeepFeature(image, convnet)

%{
RESIZE:
1. short side is at least 256 pixels
2. image area is almost the same
3. short and long size must be 32x numbers
4. WARP if not a multiplier of 32
5. short and long: WARP together (sqrt form)

ALGO:
1. pre-define an image size: 512x512 (16x16 at conv5)
2. compute the conv5 size of each image (at least 8x or x8)
>find a best match among the following choices:
  {8x32, 9x28, 10x26, 11x23, 12x21, 13x20, 14x18, 15x17, 16x16}
3. go back to the original size (multiply by 32)
4. resize with WARP (same ratio at width and height axis)
%}

AREA = 256;
MIN_RESIZE = 8;
MAX_RESIZE = 16;
SCALE = 32;

[height, width, ~] = size(image);
def = ones(1, MAX_RESIZE) * (AREA / MIN_RESIZE / MIN_RESIZE);
if (height <= width)
    ratio = width / height;
    for h = MIN_RESIZE: 1: MAX_RESIZE
        w = round(AREA / h);
        r = w / h;
        def(h) = max(r / ratio, ratio / r);
    end
    [~, h] = min(def);
    w = round(AREA / h);
    image = imresize(image, [h * SCALE, w * SCALE]);
else
    ratio = height / width;
    for w = MIN_RESIZE: 1: MAX_RESIZE
        h = round(AREA / w);
        r = h / w;
        def(w) = max(r / ratio, ratio / r);
    end
    [~, w] = min(def);
    h = round(AREA / w);
    image = imresize(image, [h * SCALE, w * SCALE]);
end

if (size(image, 3) == 1)
    image = repmat(image, [1, 1, 3]);
end
image = single(image);
[height, width, ~] = size(image);
averageImage = imresize(convnet.normalization.averageImage, [height, width]);
image = image - averageImage;
result = vl_simplenn_s(convnet, image, 1, [], 41, 1, ...
    'accumulate', false, 'disableDropout', true);

end
