function finished = ExtractConvnet(convnetFile, imagePath, descriptorPath, gradientPath)

convnet = load(convnetFile);

feature = [];
feature.categories = 0;
feature.images = 0;

if (exist(imagePath, 'dir'))
    fprintf('Begin to extract ConvNet features.\n');
    categoryDirectories = dir(imagePath);
    for i = 1: 1: length(categoryDirectories)
        categoryName = categoryDirectories(i).name;
        if (~strcmp(categoryName, '.') && ~strcmp(categoryName, '..'))
            feature.categories = feature.categories + 1;
            fprintf('Category # %03d: %s, begins.\n', feature.categories, categoryName);
            ca = feature.categories;
            feature.cate(ca).images = 0;
            imageDirectory = fullfile(imagePath, categoryName);
            imageFiles = dir(fullfile(imageDirectory, '*.jpg'));
            for j = 1: 1: length(imageFiles)
                feature.cate(ca).images = feature.cate(ca).images + 1;
                imageFile = fullfile(imageDirectory, imageFiles(j).name);
                image = imread(imageFile);
                result = DeepFeature(image, convnet);
                for layer = [28, 30, 32, 34, 37, 39]
                %for layer = [0, 2, 5, 7, 10, 12, 14, 16, 19, 21, 23, 25, 28, 30, 32, 34, 37, 39]
                    [~, name_] = fileparts(imageFiles(j).name);
                    descriptor1Path_ = [descriptorPath, ',', num2str(layer), 'L'];
                    descriptor1Directory = fullfile(descriptor1Path_, categoryName);
                    if (~isdir(descriptor1Directory))
                        mkdir(descriptor1Directory);
                    end
                    descriptor1File = fullfile(descriptor1Directory, [name_, '.mat']);
                    if (~exist(descriptor1File, 'file'))
                        descriptors = gather(result(layer + 1).x); %#ok<NASGU>
                        save(descriptor1File, 'descriptors');
                    end
                    descriptor2Path_ = [descriptorPath, ',', num2str(layer + 2), 'L'];
                    descriptor2Directory = fullfile(descriptor2Path_, categoryName);
                    if (~isdir(descriptor2Directory))
                        mkdir(descriptor2Directory);
                    end
                    descriptor2File = fullfile(descriptor2Directory, [name_, '.mat']);
                    descriptors = gather(result(layer + 3).x); %#ok<NASGU>
                    save(descriptor2File, 'descriptors');
                    gradientPath_ = [gradientPath, ',', num2str(layer + 2), 'L'];
                    gradientDirectory = fullfile(gradientPath_, categoryName);
                    if (~isdir(gradientDirectory))
                        mkdir(gradientDirectory);
                    end
                    gradientFile = fullfile(gradientDirectory, [name_, '.mat']);
                    gradients = gather(result(layer + 3).dzdx); %#ok<NASGU>
                    save(gradientFile, 'gradients');
                end
                fprintf('    Image #%03d out of %d images of category %03d is finished.\n', ...
                    j, length(imageFiles), ca);
            end
            feature.images = feature.images + feature.cate(ca).images;
            fprintf('Category # %03d: %s, is finished, %04d images have been processed.\n', ...
                feature.categories, categoryName, feature.images);
        end
    end
    fprintf('Extraction of ConvNet features is finished.\n');
    finished = false;
else
    fprintf('Bad image directory. Please check if %s exists.\n', imagePath);
    finished = false;
end

end
